@@ -24,12 +24,28 @@ SoftmaxErrorFunction<MetricType>::SoftmaxErrorFunction(
     const arma::mat& dataset,
     const arma::Row<size_t>& labels,
     MetricType metric) :
-    dataset(dataset),
-    labels(labels),
+    dataset(math::MakeAlias(const_cast<arma::mat&>(dataset), false)),
+    labels(math::MakeAlias(const_cast<arma::Row<size_t>&>(labels), false)),
     metric(metric),
     precalculated(false)
 { /* nothing to do */ }
 
+//! Shuffle the dataset.
+template<typename MetricType>
+void SoftmaxErrorFunction<MetricType>::Shuffle()
+{
+  arma::mat newDataset;
+  arma::Row<size_t> newLabels;
+
+  math::ShuffleData(dataset, labels, newDataset, newLabels);
+
+  math::ClearAlias(dataset);
+  math::ClearAlias(labels);
+
+  dataset = std::move(newDataset);
+  labels = std::move(newLabels);
+}
+
 //! The non-separable implementation, which uses Precalculate() to save time.
 template<typename MetricType>
 double SoftmaxErrorFunction<MetricType>::Evaluate(const arma::mat& coordinates)
@@ -41,46 +57,52 @@ double SoftmaxErrorFunction<MetricType>::Evaluate(const arma::mat& coordinates)
                    // minimizes, not maximizes.
 };
 
-//! The separated objective function, which does not use Precalculate().
+//! The separated objective function, which does not use Precalculate(),
+//! for a given batch size and from an initial index.
 template<typename MetricType>
 double SoftmaxErrorFunction<MetricType>::Evaluate(const arma::mat& coordinates,
-                                                  const size_t i)
+                                                  const size_t begin,
+                                                  const size_t batchSize)
 {
   // Unfortunately each evaluation will take O(N) time because it requires a
   // scan over all points in the dataset.  Our objective is to compute p_i.
   double denominator = 0;
   double numerator = 0;
+  double result = 0;
 
   // It's quicker to do this now than one point at a time later.
   stretchedDataset = coordinates * dataset;
-
-  for (size_t k = 0; k < dataset.n_cols; ++k)
+  for (size_t i = begin; i < begin + batchSize; i++)
   {
-    // Don't consider the case where the points are the same.
-    if (k == i)
+    for (size_t k = 0; k < dataset.n_cols; ++k)
+    {
+      // Don't consider the case where the points are the same.
+      if (k == i)
+        continue;
+
+      // We want to evaluate exp(-D(A x_i, A x_k)).
+      double eval = std::exp(-metric.Evaluate(stretchedDataset.unsafe_col(i),
+                                              stretchedDataset.unsafe_col(k)));
+
+      // If they are in the same class, update the numerator.
+      if (labels[i] == labels[k])
+        numerator += eval;
+
+      denominator += eval;
+    }
+
+    // Now the result is just a simple division, but we have to be sure that the
+    // denominator is not 0.
+    if (denominator == 0.0)
+    {
+      Log::Warn << "Denominator of p_" << i << " is 0!" << std::endl;
       continue;
+    }
 
-    // We want to evaluate exp(-D(A x_i, A x_k)).
-    double eval = std::exp(-metric.Evaluate(stretchedDataset.unsafe_col(i),
-                                            stretchedDataset.unsafe_col(k)));
-
-    // If they are in the same class, update the numerator.
-    if (labels[i] == labels[k])
-      numerator += eval;
-
-    denominator += eval;
+    result += -(numerator / denominator); // Negate because the optimizer is a
+                                          // minimizer.
   }
-
-  // Now the result is just a simple division, but we have to be sure that the
-  // denominator is not 0.
-  if (denominator == 0.0)
-  {
-    Log::Warn << "Denominator of p_" << i << " is 0!" << std::endl;
-    return 0;
-  }
-
-  return -(numerator / denominator); // Negate because the optimizer is a
-                                     // minimizer.
+  return result;
 }
 
 //! The non-separable implementation, where Precalculate() is used.
@@ -131,74 +153,80 @@ void SoftmaxErrorFunction<MetricType>::Gradient(const arma::mat& coordinates,
   gradient = -2 * coordinates * sum;
 }
 
-//! The separable implementation.
-template<typename MetricType>
+//! The separable implementation for a given batch size and an initial index.
+template <typename MetricType>
+template <typename GradType>
 void SoftmaxErrorFunction<MetricType>::Gradient(const arma::mat& coordinates,
-                                                const size_t i,
-                                                arma::mat& gradient)
+                                                const size_t begin,
+                                                GradType& gradient,
+                                                const size_t batchSize)
 {
-  // We will need to calculate p_i before this evaluation is done, so these two
-  // variables will hold the information necessary for that.
-  double numerator = 0;
-  double denominator = 0;
-
   // The gradient involves two matrix terms which are eventually combined into
   // one.
-  arma::mat firstTerm;
-  arma::mat secondTerm;
+  GradType firstTerm, secondTerm;
+  // We will need to calculate p_i before this evaluation is done, so
+  // these two variables will hold the information necessary for that.
+  double numerator, denominator;
 
-  firstTerm.zeros(coordinates.n_rows, coordinates.n_cols);
-  secondTerm.zeros(coordinates.n_rows, coordinates.n_cols);
+  gradient.zeros(coordinates.n_rows, coordinates.n_rows);
 
   // Compute the stretched dataset.
   stretchedDataset = coordinates * dataset;
-
-  for (size_t k = 0; k < dataset.n_cols; ++k)
+  for (size_t i = begin; i < begin + batchSize; i++)
   {
-    // Don't consider the case where the points are the same.
-    if (i == k)
-      continue;
+    numerator = 0;
+    denominator = 0;
 
-    // Calculate the numerator of p_ik.
-    double eval = exp(-metric.Evaluate(stretchedDataset.unsafe_col(i),
-                                       stretchedDataset.unsafe_col(k)));
+    firstTerm.zeros(coordinates.n_rows, coordinates.n_cols);
+    secondTerm.zeros(coordinates.n_rows, coordinates.n_cols);
 
-    // If the points are in the same class, we must add to the second term of
-    // the gradient as well as the numerator of p_i.  We will divide by the
-    // denominator of p_ik later.  For x_ik we are not using stretched points.
-    arma::vec x_ik = dataset.col(i) - dataset.col(k);
-    if (labels[i] == labels[k])
+    for (size_t k = 0; k < dataset.n_cols; ++k)
     {
-      numerator += eval;
-      secondTerm += eval * x_ik * trans(x_ik);
+      // Don't consider the case where the points are the same.
+      if (i == k)
+        continue;
+
+      // Calculate the numerator of p_ik.
+      double eval = exp(-metric.Evaluate(stretchedDataset.unsafe_col(i),
+                                         stretchedDataset.unsafe_col(k)));
+
+      // If the points are in the same class, we must add to the second term of
+      // the gradient as well as the numerator of p_i.  We will divide by the
+      // denominator of p_ik later.  For x_ik we are not using stretched points.
+      GradType x_ik = dataset.col(i) - dataset.col(k);
+      if (labels[i] == labels[k])
+      {
+        numerator += eval;
+        secondTerm += eval * x_ik * trans(x_ik);
+      }
+
+      // We always have to add to the denominator of p_i
+      // and the first term of the gradient computation.
+      // We will divide by the denominator of p_ik later.
+      denominator += eval;
+      firstTerm += eval * x_ik * trans(x_ik);
     }
 
-    // We always have to add to the denominator of p_i and the first term of the
-    // gradient computation.  We will divide by the denominator of p_ik later.
-    denominator += eval;
-    firstTerm += eval * x_ik * trans(x_ik);
-  }
+    // Calculate p_i.
+    double p = 0;
+    if (denominator == 0)
+    {
+      Log::Warn << "Denominator of p_" << i << " is 0!" << std::endl;
+      // If the denominator is zero, then all p_ik should be zero and there is
+      // no gradient contribution from this point.
+      continue;
+    }
+    else
+    {
+      p = numerator / denominator;
+      firstTerm /= denominator;
+      secondTerm /= denominator;
+    }
 
-  // Calculate p_i.
-  double p = 0;
-  if (denominator == 0)
-  {
-    Log::Warn << "Denominator of p_" << i << " is 0!" << std::endl;
-    // If the denominator is zero, then all p_ik should be zero and there is
-    // no gradient contribution from this point.
-    gradient.zeros(coordinates.n_rows, coordinates.n_rows);
-    return;
+    // Now multiply the first term by p_i, and add the two together and multiply
+    // all by 2 * A.  We negate it though, because our optimizer is a minimizer.
+    gradient += -2 * coordinates * (p * firstTerm - secondTerm);
   }
-  else
-  {
-    p = numerator / denominator;
-    firstTerm /= denominator;
-    secondTerm /= denominator;
-  }
-
-  // Now multiply the first term by p_i, and add the two together and multiply
-  // all by 2 * A.  We negate it though, because our optimizer is a minimizer.
-  gradient = -2 * coordinates * (p * firstTerm - secondTerm);
 }
 
 template<typename MetricType>
