@@ -32,15 +32,15 @@ BOOST_AUTO_TEST_CASE(CFGetRecommendationsAllUsersTest)
 {
   // Dummy number of recommendations.
   size_t numRecs = 3;
-  // GroupLens100k.csv dataset has 943 users.
-  size_t numUsers = 943;
+  // GroupLensSmall.csv dataset has 200 users.
+  size_t numUsers = 200;
 
   // Matrix to save recommendations into.
   arma::Mat<size_t> recommendations;
 
   // Load GroupLens data.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   // Make data into sparse matrix.
   arma::sp_mat cleanedData;
@@ -80,7 +80,7 @@ BOOST_AUTO_TEST_CASE(CFGetRecommendationsQueriedUserTest)
 
   // Load GroupLens data.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   // Make data into sparse matrix.
   arma::sp_mat cleanedData;
@@ -105,14 +105,15 @@ BOOST_AUTO_TEST_CASE(RecommendationAccuracyTest)
 {
   // Load the GroupLens dataset; then, we will remove some values from it.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 300); // Remove 300 5-star ratings.
+  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
+  savedCols.fill(/* random very large value */ 10000000);
   size_t currentCol = 0;
   for (size_t i = 0; i < dataset.n_cols; ++i)
   {
-    if (currentCol == 300)
+    if (currentCol == 50)
       break;
 
     if (dataset(2, i) > 4.5) // 5-star rating.
@@ -150,18 +151,18 @@ BOOST_AUTO_TEST_CASE(RecommendationAccuracyTest)
   // Obtain 150 recommendations for the users in savedCols, and make sure the
   // missing item shows up in most of them.  First, create the list of users,
   // which requires casting from doubles...
-  arma::Col<size_t> users(300);
-  for (size_t i = 0; i < 300; ++i)
+  arma::Col<size_t> users(50);
+  for (size_t i = 0; i < 50; ++i)
     users(i) = (size_t) savedCols(0, i);
   arma::Mat<size_t> recommendations;
   size_t numRecs = 150;
   c.GetRecommendations(numRecs, recommendations, users);
 
   BOOST_REQUIRE_EQUAL(recommendations.n_rows, numRecs);
-  BOOST_REQUIRE_EQUAL(recommendations.n_cols, 300);
+  BOOST_REQUIRE_EQUAL(recommendations.n_cols, 50);
 
   size_t failures = 0;
-  for (size_t i = 0; i < 300; ++i)
+  for (size_t i = 0; i < 50; ++i)
   {
     size_t targetItem = (size_t) savedCols(1, i);
     bool found = false;
@@ -186,11 +187,11 @@ BOOST_AUTO_TEST_CASE(RecommendationAccuracyTest)
       ++failures;
   }
 
-  // Make sure the right item showed up in at least 2/3 of the recommendations.
-  // Random chance (that is, if we selected recommendations randomly) for this
+  // Make sure the right item showed up in at least 1/3 of the recommendations.
   // GroupLens dataset would give somewhere around a 10% success rate (failures
-  // would be closer to 270).
-  BOOST_REQUIRE_LT(failures, 100);
+  // would be closer to 270).  The failure rate is allowed to be so high because
+  // the dataset used here is pretty small and it is hard to generalize.
+  BOOST_REQUIRE_LT(failures, 17);
 }
 
 // Make sure that Predict() is returning reasonable results.
@@ -198,14 +199,15 @@ BOOST_AUTO_TEST_CASE(CFPredictTest)
 {
   // Load the GroupLens dataset; then, we will remove some values from it.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 300); // Remove 300 5-star ratings.
+  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
+  savedCols.fill(/* random very large value */ 10000000);
   size_t currentCol = 0;
   for (size_t i = 0; i < dataset.n_cols; ++i)
   {
-    if (currentCol == 300)
+    if (currentCol == 50)
       break;
 
     if (dataset(2, i) > 4.5) // 5-star rating.
@@ -264,14 +266,15 @@ BOOST_AUTO_TEST_CASE(CFBatchPredictTest)
 {
   // Load the GroupLens dataset; then, we will remove some values from it.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 300); // Remove 300 5-star ratings.
+  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
+  savedCols.fill(/* random very large value */ 10000000);
   size_t currentCol = 0;
   for (size_t i = 0; i < dataset.n_cols; ++i)
   {
-    if (currentCol == 300)
+    if (currentCol == 50)
       break;
 
     if (dataset(2, i) > 4.5) // 5-star rating.
@@ -336,14 +339,15 @@ BOOST_AUTO_TEST_CASE(TrainTest)
 
   // Now retrain with data we know about.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 300); // Remove 300 5-star ratings.
+  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
+  savedCols.fill(/* random very large value */ 10000000);
   size_t currentCol = 0;
   for (size_t i = 0; i < dataset.n_cols; ++i)
   {
-    if (currentCol == 300)
+    if (currentCol == 50)
       break;
 
     if (dataset(2, i) > 4.5) // 5-star rating.
@@ -406,14 +410,15 @@ BOOST_AUTO_TEST_CASE(EmptyConstructorTrainTest)
 
   // Now retrain with data we know about.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 300); // Remove 300 5-star ratings.
+  arma::mat savedCols(3, 50); // Remove 300 5-star ratings.
+  savedCols.fill(/* random very large value */ 10000000);
   size_t currentCol = 0;
   for (size_t i = 0; i < dataset.n_cols; ++i)
   {
-    if (currentCol == 300)
+    if (currentCol == 50)
       break;
 
     if (dataset(2, i) > 4.5) // 5-star rating.
@@ -473,7 +478,7 @@ BOOST_AUTO_TEST_CASE(SerializationTest)
 {
   // Load a dataset to train on.
   arma::mat dataset;
-  data::Load("GroupLens100k.csv", dataset);
+  data::Load("GroupLensSmall.csv", dataset);
 
   arma::sp_mat cleanedData;
   CF::CleanData(dataset, cleanedData);
